@@ -3,7 +3,7 @@ Tools for drawing Python object reference graphs with graphviz.
 
 You can find documentation online at http://mg.pov.lt/objgraph/
 
-Copyright (c) 2008-2015 Marius Gedminas <marius@pov.lt> and contributors
+Copyright (c) 2008-2016 Marius Gedminas <marius@pov.lt> and contributors
 
 Released under the MIT licence.
 """
@@ -46,10 +46,10 @@ except ImportError:
 
 
 __author__ = "Marius Gedminas (marius@gedmin.as)"
-__copyright__ = "Copyright (c) 2008-2015 Marius Gedminas and contributors"
+__copyright__ = "Copyright (c) 2008-2016 Marius Gedminas and contributors"
 __license__ = "MIT"
-__version__ = "2.0.1"
-__date__ = "2015-07-28"
+__version__ = "3.0.0"
+__date__ = "2016-04-13"
 
 
 try:
@@ -65,19 +65,42 @@ except AttributeError:
     iteritems = dict.items
 
 
+def _isinstance(object, classinfo):
+    """Return whether an object is an instance of a class or its subclass.
+
+    Differs from the builtin isinstance() implementation in that it does not
+    depend on the ``__class__`` attribute which is proxied by
+    mock.Mock(spec=...).
+    """
+    return issubclass(type(object), classinfo)
+
+
 def count(typename, objects=None):
     """Count objects tracked by the garbage collector with a given class name.
 
+    The class name can optionally be fully qualified.
+
     Example:
 
         >>> count('dict')
         42
-        >>> count('MyClass', get_leaking_objects())
-        3
         >>> count('mymodule.MyClass')
         2
 
-    Note that the GC does not track simple objects like int or str.
+    .. note::
+
+        The Python garbage collector does not track simple
+        objects like int or str.  See
+        https://docs.python.org/3/library/gc.html#gc.is_tracked
+        for more information.
+
+    Instead of looking through all objects tracked by the GC, you may
+    specify your own collection, e.g.
+
+        >>> count('MyClass', get_leaking_objects())
+        3
+
+    See also: :func:`get_leaking_objects`.
 
     .. versionchanged:: 1.7
        New parameter: ``objects``.
@@ -169,7 +192,11 @@ def most_common_types(limit=10, objects=None, shortnames=True):
     return stats
 
 
-def show_most_common_types(limit=10, objects=None, shortnames=True):
+def show_most_common_types(
+        limit=10,
+        objects=None,
+        shortnames=True,
+        file=sys.stdout):
     """Print the table of types of most common instances.
 
     The caveats documented in :func:`typestats` apply.
@@ -191,14 +218,17 @@ def show_most_common_types(limit=10, objects=None, shortnames=True):
     .. versionchanged:: 1.8
        New parameter: ``shortnames``.
 
+    .. versionchanged:: 2.1
+       New parameter: ``file``.
+
     """
     stats = most_common_types(limit, objects, shortnames=shortnames)
     width = max(len(name) for name, count in stats)
     for name, count in stats:
-        print('%-*s %i' % (width, name, count))
+        file.write('%-*s %i\n' % (width, name, count))
 
 
-def show_growth(limit=10, peak_stats={}, shortnames=True):
+def show_growth(limit=10, peak_stats={}, shortnames=True, file=sys.stdout):
     """Show the increase in peak object counts since last call.
 
     Limits the output to ``limit`` largest deltas.  You may set ``limit`` to
@@ -223,6 +253,9 @@ def show_growth(limit=10, peak_stats={}, shortnames=True):
     .. versionchanged:: 1.8
        New parameter: ``shortnames``.
 
+    .. versionchanged:: 2.1
+       New parameter: ``file``.
+
     """
     gc.collect()
     stats = typestats(shortnames=shortnames)
@@ -239,7 +272,7 @@ def show_growth(limit=10, peak_stats={}, shortnames=True):
     if deltas:
         width = max(len(name) for name, count in deltas)
         for name, delta in deltas:
-            print('%-*s%9d %+9d' % (width, name, stats[name], delta))
+            file.write('%-*s%9d %+9d\n' % (width, name, stats[name], delta))
 
 
 def get_leaking_objects(objects=None):
@@ -615,7 +648,7 @@ def _show_graph(objs, edge_func, swap_source_target,
                 highlight=None, filename=None, extra_info=None,
                 refcounts=False, shortnames=True, output=None,
                 cull_func=None):
-    if not isinstance(objs, (list, tuple)):
+    if not _isinstance(objs, (list, tuple)):
         objs = [objs]
     if filename and output:
         raise ValueError('Cannot specify both output and filename.')
@@ -834,26 +867,31 @@ def _safe_repr(obj):
         return '(unrepresentable)'
 
 
-def _short_repr(obj):
-    if isinstance(obj, (type, types.ModuleType, types.BuiltinMethodType,
-                        types.BuiltinFunctionType)):
-        return obj.__name__
-    if isinstance(obj, types.MethodType):
-        try:
-            if obj.__self__ is not None:
-                return obj.__func__.__name__ + ' (bound)'
-            else:
-                return obj.__func__.__name__
-        except AttributeError:  # pragma: nocover
-            # Python < 2.6 compatibility
-            if obj.im_self is not None:
-                return obj.im_func.__name__ + ' (bound)'
-            else:
-                return obj.im_func.__name__
+def _name_or_repr(value):
+    try:
+        result = value.__name__
+    except AttributeError:
+        result = repr(value)[:40]
 
-    if isinstance(obj, types.FrameType):
+    if _isinstance(result, basestring):
+        return result
+    else:
+        return repr(value)[:40]
+
+
+def _short_repr(obj):
+    if _isinstance(obj, (type, types.ModuleType, types.BuiltinMethodType,
+                         types.BuiltinFunctionType)):
+        return _name_or_repr(obj)
+    if _isinstance(obj, types.MethodType):
+        name = _name_or_repr(obj.__func__)
+        if obj.__self__:
+            return name + ' (bound)'
+        else:
+            return name
+    if _isinstance(obj, types.FrameType):
         return '%s:%s' % (obj.f_code.co_filename, obj.f_lineno)
-    if isinstance(obj, (tuple, list, dict, set)):
+    if _isinstance(obj, (tuple, list, dict, set)):
         return '%d items' % len(obj)
     return repr(obj)[:40]
 
@@ -872,15 +910,15 @@ def _gradient(start_color, end_color, depth, max_depth):
 
 
 def _edge_label(source, target, shortnames=True):
-    if (isinstance(target, dict) and
+    if (_isinstance(target, dict) and
             target is getattr(source, '__dict__', None)):
         return ' [label="__dict__",weight=10]'
-    if isinstance(source, types.FrameType):
+    if _isinstance(source, types.FrameType):
         if target is source.f_locals:
             return ' [label="f_locals",weight=10]'
         if target is source.f_globals:
             return ' [label="f_globals",weight=10]'
-    if isinstance(source, types.MethodType):
+    if _isinstance(source, types.MethodType):
         try:
             if target is source.__self__:
                 return ' [label="__self__",weight=10]'
@@ -892,14 +930,14 @@ def _edge_label(source, target, shortnames=True):
                 return ' [label="im_self",weight=10]'
             if target is source.im_func:
                 return ' [label="im_func",weight=10]'
-    if isinstance(source, types.FunctionType):
+    if _isinstance(source, types.FunctionType):
         for k in dir(source):
             if target is getattr(source, k):
                 return ' [label="%s",weight=10]' % _quote(k)
-    if isinstance(source, dict):
+    if _isinstance(source, dict):
         for k, v in iteritems(source):
             if v is target:
-                if isinstance(k, basestring) and _is_identifier(k):
+                if _isinstance(k, basestring) and _is_identifier(k):
                     return ' [label="%s",weight=2]' % _quote(k)
                 else:
                     if shortnames:
